"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Sequelize = require("sequelize");
const path = require("path");
const lodash_1 = require("lodash");
const iface_1 = require("../utils/iface");
const send = require("koa-send");
const errors_1 = require("../utils/errors");
const process = require("process");
var connection = null;
class Config {
    constructor(ctx) {
        this.sendFile = false;
        this._transTimes = 0;
        if (ctx) {
            this.controller(ctx);
            this._ctx = ctx;
        }
    }
    get AppDebug() {
        return process.env.NODE_ENV == "production";
    }
    get AppPath() {
        return './dist';
    }
    get session() {
        return {
            key: 'castlekoa',
            path: '.sess',
            maxAge: 86400000,
            overwrite: true,
            httpOnly: true,
            signed: true,
            rolling: false,
            renew: false,
        };
    }
    get dbConfig() {
        return {
            database: 'test',
            username: 'root',
            password: '123456',
            options: {
                host: 'localhost',
                port: 3306,
                dialect: 'mysql',
                operatorsAliases: iface_1.DbOp,
                timezone: '+8:00',
                pool: { max: 5, min: 1, acquire: 3000, idle: 1000 },
                logging: (content) => {
                    console.log(content);
                }
            },
        };
    }
    get dbPrefix() {
        return '';
    }
    get db() {
        if (!connection) {
            connection = new Sequelize(this.dbConfig.database, this.dbConfig.username, this.dbConfig.password, Object.assign(this.dbConfig.options, { operatorsAliases: iface_1.DbOp }));
        }
        return connection;
    }
    async startTrans() {
        this._transTimes++;
        if (this._trans) {
            return this._trans;
        }
        this._trans = await this.db.transaction();
        return this._trans;
    }
    async commit() {
        this._transTimes--;
        if (this._trans && this._transTimes == 0) {
            await this._trans.commit();
            this._trans = undefined;
            return true;
        }
    }
    async rollback() {
        this._transTimes--;
        if (this._trans && this._transTimes == 0) {
            await this._trans.rollback();
            this._trans = undefined;
            return true;
        }
    }
    get checkProxy() {
        return false;
    }
    controller(ctx) {
        let path = ctx.path.split('/');
        if (path.length > 1) {
            if (path[0] == "" && path.length > 2) {
                path.shift();
            }
            ctx.control = {
                c: path[0],
                m: path.length == 1 ? 'index' : path[1],
            };
        }
    }
    getDbDefine(TableName) {
        try {
            let d = require(path.resolve(path.join(this.AppPath, 'db', TableName)));
            return d.default;
        }
        catch (e) {
            throw new Error(`${errors_1.default.DB_DEFINE_NOT_FOUND}_${TableName}`);
        }
    }
    getDbTableFields(TableName) {
        return this.getDbDefine(TableName);
    }
    getDbTablePK(TableName) {
        let pk = "";
        lodash_1.forOwn(this.getDbTableFields(TableName), (config, field) => {
            if (config.primaryKey)
                pk = field;
        });
        return pk;
    }
    getRelation(RelationName) {
        return false;
    }
    get StaticPath() {
        return path.join(path.dirname(this.AppPath), 'Public');
    }
    async getStaticFile(p) {
        if (p === undefined) {
            p = path.join(this.StaticPath, this._ctx.path);
        }
        await send(this._ctx, this._ctx.path, {
            root: this.StaticPath,
            index: 'index.html',
            maxage: 86400000,
        });
    }
    get allowCORS() {
        return true;
    }
    get outformat() {
        return {
            d: this._ctx.body !== undefined ? this._ctx.body : '',
            c: this._ctx.error ? 500 : 200,
            i: [this._ctx.control.c, this._ctx.control.m].join('/'),
            e: this._ctx.error ? this._ctx.error : ''
        };
    }
}
exports.default = Config;
//# sourceMappingURL=index.js.map