"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const Sequelize = require("sequelize");
const errors_1 = require("../utils/errors");
exports.Op = Sequelize.Op;
exports.QueryTypes = Sequelize.QueryTypes;
var Operate;
(function (Operate) {
    Operate[Operate["Select"] = 0] = "Select";
    Operate[Operate["Add"] = 1] = "Add";
    Operate[Operate["Delete"] = 2] = "Delete";
    Operate[Operate["Save"] = 3] = "Save";
})(Operate = exports.Operate || (exports.Operate = {}));
class Model {
    constructor(ctx, Table, Prefix = "") {
        this._true_table_name = "";
        this._fields = [];
        this._table_name = "";
        this._options = {
            fields: [], where: {}, group: []
        };
        this._config = {};
        this._operate = Operate.Select;
        this.$search = {};
        this.object = function () {
        };
        this._clean = function () {
            this._options.fields = [];
            this._options.where = {};
            if (!_.isUndefined(this._options.limit))
                delete this._options.limit;
            if (!_.isUndefined(this._options.offset))
                delete this._options.offset;
        };
        this._ctx = ctx;
        this._table_name = Table;
        this._true_table_name = Prefix + Table.replace(/([A-Z])/g, function ($0, $1) {
            return '_' + $1.toLowerCase();
        }).replace(/_(.+)/, "$1");
        this._db = this._ctx.config.db;
    }
    static parseWhere(where) {
        var w = {};
        _.forOwn(where, (v, k) => {
            if ("string" == typeof k) {
                if (Sequelize.Op[k]) {
                    if (v instanceof Array) {
                        w[Sequelize.Op[k]] = v;
                    }
                    else if ('object' == typeof v) {
                        w[Sequelize.Op[k]] = Model.parseWhere(v);
                    }
                    else
                        w[Sequelize.Op[k]] = v;
                }
                else {
                    w[k] = v;
                }
            }
            else {
                w[k] = v;
            }
        });
        return w;
    }
    get changeOptions() {
        let d = {};
        if (this.transaction) {
            d.transaction = this.transaction;
        }
        return d;
    }
    get config() {
        return this._config;
    }
    set config(config) {
        this._config = config;
    }
    _parse_where() {
        return Model.parseWhere(this._options.where);
    }
    _parse_fields() {
        if (this._options.fields.length == 0) {
            return this._ctx.config.getDbTableFields(this._table_name);
        }
        return this._options.fields;
    }
    _parse_order() {
        var order = [];
        if ("string" == typeof this._options.order) {
            this._options.order = this._options.order.split(',');
        }
        if (this._options.order instanceof Array) {
            _.forOwn(this._options.order, (v, k) => {
                order.push(v.split(' '));
            });
        }
        return order;
    }
    _parse_config() {
        let config = {};
        config['attributes'] = this._parse_fields();
        config['where'] = this._parse_where();
        if (this._options.order) {
            config['order'] = this._parse_order();
        }
        if (this._options.limit) {
            config['limit'] = this._options.limit;
        }
        if (this._options.offset) {
            config['offset'] = this._options.offset;
        }
        if (this._options.group && this._options.group.length > 0) {
            config['group'] = this._options.group.join(',');
        }
        if (this._operate == Operate.Select && !this._options.fields) {
            config.fields = Object.keys(this._ctx.config.getDbTableFields(this._table_name));
        }
        if (this.transaction) {
            config.transaction = this.transaction;
        }
        return config;
    }
    setTableFields(fields) {
        this._fields = fields;
    }
    define(config = {}) {
        return this._ctx.config.getDbDefine(this._table_name);
    }
    getModel() {
        if (!this._model) {
            this._model = this._db.models[this._true_table_name] ? this._db.models[this._true_table_name] : this._db.define(this._true_table_name, this.define(), {
                freezeTableName: true,
                timestamps: false
            });
        }
        return this._model;
    }
    async addIfNotExist(data, where = null) {
        let d = await this.where(where || data).find();
        if (_.isObject(d)) {
            return true;
        }
        else {
            return await this.add(data);
        }
    }
    async setDec(config) {
        if (Object.keys(config).length > 0) {
        }
        throw new Error(errors_1.default.CONFIG_ERROR);
    }
    async setInc(config) {
        throw new Error(errors_1.default.CONFIG_ERROR);
    }
    where(where) {
        if (null == where) {
            this._options.where = {};
        }
        else if (_.isObject(where))
            this._options.where = Object.assign(this._options.where, where);
        else {
            console.log('Error Where', where);
        }
        return this;
    }
    fields(fields, exclude = false) {
        if (_.isArray(fields)) {
            this._options.fields = _.concat(this._options.fields, fields);
        }
        else if (_.isString(fields)) {
            this._options.fields = _.concat(this._options.fields, fields.split(','));
        }
        return this;
    }
    order(order) {
        this._options.order = order;
        return this;
    }
    async select() {
        this._operate = Operate.Select;
        let d = await this.getModel().findAll(this._parse_config());
        let data = [];
        this._clean();
        d.forEach(v => {
            data.push(v.dataValues);
        });
        return data;
    }
    async add(data) {
        this._operate = Operate.Add;
        let d = await this.getModel().create(data, this.changeOptions);
        this._clean();
        return d.dataValues;
    }
    data(data) {
        this.getModel().build(data);
        return this;
    }
    async find() {
        this._operate = Operate.Select;
        let d = await this.limit(1).select();
        this._clean();
        return d[0];
    }
    async addAll(data) {
        this._operate = Operate.Add;
        let d = await this.getModel().bulkCreate(data, Object.assign({
            fields: Object.keys(data[0])
        }, this.changeOptions));
        this._clean();
        let ds = [];
        d.forEach(v => {
            ds.push(v.dataValues);
        });
        return ds;
    }
    async count() {
        return 0;
    }
    async selectAndCount() {
        this._operate = Operate.Select;
        let d = await this.getModel().findAndCountAll(this._parse_config());
        let data = [];
        d.rows.forEach(v => {
            data.push(v.dataValues);
        });
        this._clean();
        return {
            count: d.count,
            rows: data
        };
    }
    limit(Number) {
        this._options.limit = Number;
        return this;
    }
    page(p, n) {
        this._options.limit = Number(n);
        this._options.offset = (p - 1) * n;
        return this;
    }
    group(fields) {
        this._options.group = fields;
        return this;
    }
    async del() {
        this._operate = Operate.Delete;
        let d = await this.getModel().destroy(Object.assign({
            where: this._parse_where(),
        }, this.changeOptions));
        this._clean();
        return d;
    }
    async save(data) {
        this._operate = Operate.Save;
        let d = await this.getModel().update(data, Object.assign({
            where: this._parse_where(),
            options: {
                returning: true
            }
        }, this.changeOptions));
        this._clean();
        return d[0];
    }
    query(sql) {
        return this._db.query(sql.replace(/__DB_PREFIX__/g, this._ctx.config.dbPrefix), {
            type: Sequelize.QueryTypes.SELECT,
            transaction: this.transaction
        });
    }
    exec(SQL, Type) {
        return this._db.query(SQL.replace(/__DB_PREFIX__/g, this._ctx.config.dbPrefix), Object.assign({ type: Type }, this.changeOptions));
    }
    async startTrans() {
        this.transaction = await this._ctx.config.db.transaction();
        return this.transaction;
    }
    setTrans(trans) {
        this.transaction = trans;
        return this;
    }
    commit() {
        this.transaction.commit();
    }
    rollback() {
        this.transaction.rollback();
    }
    async getFields(Fields, More = false) {
        this._operate = Operate.Select;
        if (!More) {
            this.page(1, 1);
        }
        if (_.isString(Fields)) {
            Fields = Fields.split(',');
        }
        if (Fields.length > 0) {
            let d = await this.fields(Fields).select();
            this._clean();
            var pk = Fields[0];
            if (d.length > 0) {
                if (More) {
                    var data = {};
                    var odata = [];
                    _.forOwn(d, (v, k) => {
                        if (Fields.length == 1) {
                            odata.push(v[pk]);
                        }
                        else {
                            data[v[pk]] = v;
                        }
                    });
                    return Fields.length == 1 ? odata : data;
                }
                else {
                    if (Fields.length == 1) {
                        return d[0][pk];
                    }
                    else {
                        return d[0];
                    }
                }
            }
            else {
                return More ? [] : '';
            }
        }
        else {
            return More ? [] : '';
        }
    }
}
exports.default = Model;
//# sourceMappingURL=model.js.map