"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bodyparser = require("koa-bodyparser");
const router_1 = require("../router");
const config_1 = require("../config");
const fs = require("fs");
const path = require("path");
const file_1 = require("../utils/file");
const iface_1 = require("../utils/iface");
const ws_1 = require("../utils/ws");
const multiparty = require("koa2-multiparty");
const session_1 = require("./session");
async function cors(ctx, next) {
    if (ctx.method == 'OPTIONS') {
        corsset(ctx);
        ctx.body = "";
    }
    else if (ctx.method == "POST") {
        if (ctx.config.allowCORS) {
            await next();
            corsset(ctx);
        }
    }
    else {
        await next();
    }
}
exports.cors = cors;
async function corsset(ctx) {
    ctx.set('Access-Control-Allow-Origin', ctx.header.origin);
    ctx.set("Access-Control-Allow-Headers", "x-requested-with, accept, origin, content-type");
    ctx.set("Access-Control-Allow-Credentials", true);
    ctx.set("Access-Control-Max-Age", 86400);
}
exports.corsset = corsset;
async function config(ctx, next) {
    let AppConfig;
    try {
        let configPath = path.resolve('dist/config/index.js');
        if (fs.existsSync(configPath)) {
            AppConfig = require(configPath).default;
        }
    }
    catch (error) {
        console.log(error);
    }
    finally {
        let c = AppConfig ? AppConfig : config_1.default;
        ctx.config = new c(ctx);
        await next();
    }
}
exports.config = config;
async function outcheck(ctx, next) {
    await next();
    if (!ctx.config.sendFile && ctx.control && !(ctx.body instanceof fs.ReadStream)) {
        if (ctx.error && ctx.status == iface_1.HttpCode.OK) {
            ctx.status = iface_1.HttpCode.SERVER_ERROR;
        }
        ctx.body = ctx.config.outformat;
    }
}
exports.outcheck = outcheck;
function middlewares(app) {
    app.use(config);
    app.use(cors);
    app.use(bodyparser({
        enableTypes: ['json', 'form', 'text',]
    }));
    app.use(multiparty());
    let configPath = path.resolve('dist/config/index.js');
    file_1.watch([configPath, 'dist/controller/*.js', 'dist/db/*.js', 'dist/relation/*.js', 'dist/utils/*.js'], [file_1.WatchType.Add, file_1.WatchType.Change, file_1.WatchType.Delete], (d) => {
        if (require.cache[path.resolve(d)]) {
            console.log(`File ${d} changed,deleted cache`);
            delete require.cache[path.resolve(d)];
        }
    });
    app.use(session_1.session);
    app.use(router_1.default);
    app.use(outcheck);
    if (app.ws) {
        app.ws.use(config);
        app.ws.use((ctx, next) => {
            setTimeout(() => {
                if (!ctx.ID) {
                    ctx.websocket.close();
                }
            }, 3000);
            ctx.websocket.on('message', async (message) => {
                if ('string' == typeof message) {
                    ctx.RPCEncoding = 'text';
                }
                ws_1.RPC.message(message, ctx);
            });
            ctx.websocket.on('close', () => {
                ws_1.RPC.close(ctx);
            });
            next();
        });
    }
}
exports.default = middlewares;
//# sourceMappingURL=index.js.map