"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path_1 = require("path");
const data_1 = require("../utils/data");
const file_1 = require("../utils/file");
const other_1 = require("../utils/other");
async function session(ctx, next) {
    if (ctx.control && ctx.control.m && ctx.control.c) {
        try {
            let sid = ctx.cookies.get(ctx.config.session.key);
            if (!sid) {
                sid = get_session_id(ctx);
                ctx.cookies.set(ctx.config.session.key, sid, {
                    maxAge: ctx.config.session.maxAge,
                    domain: ctx.config.session.domain,
                });
            }
            if (ctx.config.session.store) {
                let sess = ctx.config.session.store;
                ctx.session = new sess(sid, ctx.config.session);
            }
            else {
                ctx.session = new Session(sid, ctx.config.session);
            }
        }
        catch (error) {
        }
        finally {
            await next();
            ctx.session.save();
        }
    }
    else {
        next();
    }
}
exports.session = session;
function get_session_id(ctx) {
    return other_1.uuid();
}
function encode(data) {
    let c = "";
    Object.keys(data).forEach((k) => {
        let b = Buffer.alloc(2);
        b[0] = k.length;
        b[1] = data_1.getDataType(data[k]);
        c += `${b.toString('hex')}${k}`;
        switch (b[1]) {
            case data_1.DataType.Boolean:
                c += data[k] ? '1' : '0';
                break;
            case data_1.DataType.String:
            case data_1.DataType.Number:
                c += data[k];
                break;
            case data_1.DataType.JSON:
                c += JSON.stringify(data[k]);
                break;
            case data_1.DataType.Buffer:
                c += data[k].toString('hex');
                break;
        }
        c += "\n";
    });
    return c;
}
function decode(data) {
    let c = data.split("\n");
    let o = {};
    c.forEach((d) => {
        let b = Buffer.from(d.substr(0, 4), 'hex');
        switch (b[1]) {
            case data_1.DataType.Boolean:
                o[d.substr(4, b[0])] = d.substr(4 + b[0], 1) == '1';
                break;
            case data_1.DataType.String:
                o[d.substr(4, b[0])] = d.substr(4 + b[0]);
            case data_1.DataType.Number:
                o[d.substr(4, b[0])] = Number(d.substr(4 + b[0]));
                break;
            case data_1.DataType.JSON:
                o[d.substr(4, b[0])] = JSON.parse(d.substr(4 + b[0]));
                break;
            case data_1.DataType.Buffer:
                o[d.substr(4, b[0])] = Buffer.from(d.substr(4 + b[0]));
                break;
        }
    });
    return o;
}
class Session {
    constructor(id, options) {
        this.change = false;
        this.id = '';
        this.path = '';
        this.mtime = 0;
        this.data = {};
        this.path = path_1.join(options.path, id);
        this.id = id;
        try {
            this.data = decode(fs.readFileSync(this.path, { encoding: 'utf8' }));
        }
        catch (error) {
        }
    }
    get(key) {
        return this.data[key];
    }
    set(key, value) {
        this.change = true;
        this.data[key] = value;
    }
    save() {
        if (this.change)
            fs.writeFile(this.path, encode(this.data), (err) => {
                if (err) {
                    file_1.mkdirs(path_1.dirname(this.path));
                    this.save();
                }
            });
    }
    destroy() {
        return new Promise((s, j) => {
            fs.unlink(this.path, (err) => {
                if (err) {
                    j(err);
                }
                else {
                    s();
                }
            });
        });
    }
}
exports.Session = Session;
//# sourceMappingURL=session.js.map