"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mqtt_1 = require("mqtt");
const errors_1 = require("./errors");
var QosType;
(function (QosType) {
    QosType[QosType["NOT_SURE"] = 0] = "NOT_SURE";
    QosType[QosType["LESS_ONE"] = 1] = "LESS_ONE";
    QosType[QosType["ONLY_ONE"] = 2] = "ONLY_ONE";
})(QosType = exports.QosType || (exports.QosType = {}));
var MqttError;
(function (MqttError) {
    MqttError["SUBSCRIBE_CALLBACK_IS_NOT_A_FUNCTION"] = "SUBSCRIBE_CALLBACK_IS_NOT_A_FUNCTION";
})(MqttError = exports.MqttError || (exports.MqttError = {}));
class Mqtt {
    constructor() {
        this.cb = {};
        this.client = false;
        this.prefix = '';
        this.uuid = '';
    }
    subscribe(Topic, QosType, Callback) {
        if (Callback instanceof Function) {
            if (this.client) {
                this.client.subscribe(Topic, QosType);
                if (Topic instanceof Array) {
                    Topic.forEach((d) => {
                        this.cb[d] = Callback;
                    });
                }
                else {
                    this.cb[Topic] = Callback;
                }
            }
            else {
            }
        }
        else {
            throw new Error(errors_1.default.MQTT_SUBSCRIBE_CALLBACK_IS_NOT_A_FUNCTION);
        }
    }
    unsubscribe(Topic) {
        this.client.unsubscribe(Topic);
        if (Topic instanceof Array) {
            Topic.forEach((d) => {
                delete this.cb[d];
            });
        }
        else {
            delete this.cb[Topic];
        }
    }
    publish(Topic, Data, All = false) {
        this.client.publish(this.prefix + Topic, JSON.stringify({
            a: All ? 1 : 0,
            d: Data,
            u: this.uuid
        }));
    }
    connect(url, prefix = "", uuid = '') {
        this.prefix = prefix;
        this.uuid = uuid;
        this.client = mqtt_1.connect(url);
        this.client.on('message', this.message);
    }
    message(Topic, payload) {
        Topic = Topic.substr(this.prefix.length);
        try {
            if (this.cb[Topic] instanceof Function) {
                let msg = JSON.parse(payload.toString());
                if (msg.a || this.uuid == msg.u)
                    this.cb[Topic](msg.d, Topic, msg);
            }
        }
        catch (e) {
        }
    }
}
exports.Mqtt = Mqtt;
//# sourceMappingURL=mqtt.js.map